"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseExecutableActionProcessor = void 0;
const models_1 = require("../../models");
const utils_1 = require("../../utils");
const typedi_1 = require("typedi");
const services_1 = require("../../services");
class BaseExecutableActionProcessor extends models_1.ActionProcessor {
    get childProcessService() {
        return typedi_1.Container.get(services_1.ChildProcessService);
    }
    /**
     * Execute shell command
     * @param executable
     * @param args
     * @param wd
     * @param options
     */
    exec(executable, args, wd, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = {
                stderr: '',
                stdout: '',
                code: -1,
            };
            try {
                const on = {};
                if (options) {
                    /* istanbul ignore else */
                    if (options.verbose || options.stdout) {
                        on.stdout = (chunk) => {
                            /* istanbul ignore else */
                            if (options.verbose) {
                                console.log(chunk.toString().trim());
                                this.snapshot.log(`stdout: ${chunk}`, false, true);
                            }
                            /* istanbul ignore else */
                            if (options.stdout) {
                                result.stdout += chunk.toString();
                            }
                        };
                    }
                    /* istanbul ignore else */
                    if (options.verbose || options.stderr) {
                        on.stderr = (chunk) => {
                            /* istanbul ignore else */
                            if (options.verbose) {
                                console.error(chunk.toString().trim());
                                this.snapshot.log(`stderr: ${chunk}`, false, true);
                            }
                            /* istanbul ignore else */
                            if (options.stderr) {
                                result.stderr += chunk.toString();
                            }
                        };
                    }
                }
                wd = wd ? utils_1.FSUtil.getAbsolutePath(wd, this.snapshot.wd) : this.snapshot.wd;
                result.code = yield this.childProcessService.exec(executable, args, wd, on);
                if (result.code !== 0) {
                    result.error = new Error(`Command ${executable} exited with non-zero code.`);
                }
            }
            catch (e) {
                /* istanbul ignore next */
                result.code = -1;
                /* istanbul ignore next */
                result.error = e;
            }
            return result;
        });
    }
    /**
     * Store execution result in context
     * @param snapshot
     * @param context
     * @param parameters
     * @param assignTo
     * @param pushTo
     * @param result
     */
    storeResult(assignTo, pushTo, result) {
        return __awaiter(this, void 0, void 0, function* () {
            const value = {
                code: result.code,
                stdout: result.stdout,
                stderr: result.stderr,
            };
            if (assignTo) {
                utils_1.ContextUtil.assignTo(this.context, this.parameters, this.snapshot, assignTo, value);
            }
            if (pushTo) {
                utils_1.ContextUtil.pushTo(this.context, this.parameters, this.snapshot, pushTo, value);
            }
            if (result.error) {
                throw result.error;
            }
        });
    }
}
exports.BaseExecutableActionProcessor = BaseExecutableActionProcessor;
//# sourceMappingURL=BaseExecutableActionProcessor.js.map