"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FunctionActionHandler = exports.FunctionActionProcessor = void 0;
const models_1 = require("../../models");
const Joi = require("joi");
const typedi_1 = require("typedi");
const services_1 = require("../../services");
const object_collider_1 = require("object-collider");
const errors_1 = require("../../errors");
class FunctionActionProcessor extends models_1.ActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return FunctionActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            const script = [
                'return async function($, env, require, cwd, ctx, secrets, parameters, iteration) {',
                this.options,
                '}',
            ].join('\n');
            const fn = new Function(script)();
            const result = yield fn(typedi_1.default.get(services_1.TemplateUtilitiesRegistry).generateUtilities(this.context, this.snapshot, this.parameters, this.snapshot.wd), process.env, require, this.context.cwd, this.context.ctx, this.context.secrets, this.parameters.parameters, this.parameters.iteration);
            if (result) {
                try {
                    Joi.assert(result, FunctionActionProcessor.functionResultValidationSchema);
                }
                catch (e) {
                    throw new errors_1.ActionError(e.details.map((d) => d.message).join('\n'), errors_1.INVALID_CONFIGURATION);
                }
                ['cwd', 'ctx', 'secrets'].forEach((field) => {
                    /* istanbul ignore else */
                    if (!object_collider_1.isMissing(result[field])) {
                        this.context[field] = result[field];
                    }
                });
                this.snapshot.setContext(this.context);
                ['parameters', 'iteration'].forEach((field) => {
                    /* istanbul ignore else */
                    if (!object_collider_1.isMissing(result[field])) {
                        this.parameters[field] = result[field];
                    }
                });
            }
        });
    }
}
exports.FunctionActionProcessor = FunctionActionProcessor;
FunctionActionProcessor.validationSchema = Joi.string().min(1).required().options({ abortEarly: true });
FunctionActionProcessor.functionResultValidationSchema = Joi.object().keys({
    cwd: Joi.string().min(1),
    ctx: Joi.object(),
    secrets: Joi.object(),
    parameters: Joi.any(),
    iteration: Joi.object().keys({
        index: Joi.number().min(0).required(),
        value: Joi.any(),
        key: Joi.string(),
    }),
});
class FunctionActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return FunctionActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new FunctionActionProcessor(options, context, snapshot, parameters);
    }
}
exports.FunctionActionHandler = FunctionActionHandler;
FunctionActionHandler.metadata = {
    id: 'com.fireblink.fbl.function',
    aliases: ['fbl.function', 'function', 'function()', 'fn', 'fn()'],
};
//# sourceMappingURL=FunctionActionHandler.js.map