"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AttachedFlowActionHandler = exports.AttachedFlowActionProcessor = void 0;
const models_1 = require("../../models");
const typedi_1 = require("typedi");
const Joi = require("joi");
const services_1 = require("../../services");
const utils_1 = require("../../utils");
class AttachedFlowActionProcessor extends models_1.ActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return AttachedFlowActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this.options === 'string') {
                this.options = {
                    path: this.options,
                };
            }
            // get absolute path
            this.options.path = utils_1.FSUtil.getAbsolutePath(this.options.path, this.snapshot.wd);
            const flowService = typedi_1.Container.get(services_1.FlowService);
            const fbl = typedi_1.Container.get(services_1.FBLService);
            this.snapshot.log(`Attaching flow at path ${this.options.path}.`);
            const flow = yield flowService.readFlowFromFile(this.options, this.context, this.snapshot, this.parameters);
            const childSnapshot = yield fbl.execute(this.options.path, flow.wd, flow.flow, this.context, this.parameters);
            this.snapshot.registerChildActionSnapshot(childSnapshot);
        });
    }
}
exports.AttachedFlowActionProcessor = AttachedFlowActionProcessor;
AttachedFlowActionProcessor.validationSchema = Joi.alternatives(Joi.string().min(1), Joi.object({
    path: Joi.string().min(1).required(),
    http: Joi.object({
        headers: Joi.object().min(1),
    }),
    target: Joi.string().min(1),
    cache: Joi.boolean(),
}))
    .required()
    .options({ abortEarly: true });
class AttachedFlowActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return AttachedFlowActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new AttachedFlowActionProcessor(options, context, snapshot, parameters);
    }
}
exports.AttachedFlowActionHandler = AttachedFlowActionHandler;
AttachedFlowActionHandler.metadata = {
    id: 'com.fireblink.fbl.flow.attachment',
    aliases: ['fbl.flow.attachment', 'flow.attachment', 'attachment', '@'],
};
//# sourceMappingURL=AttachedFlowActionHandler.js.map