"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ParallelFlowActionHandler = exports.ParallelFlowActionProcessor = void 0;
const models_1 = require("../../models");
const typedi_1 = require("typedi");
const Joi = require("joi");
const services_1 = require("../../services");
const schemas_1 = require("../../schemas");
const BaseFlowActionProcessor_1 = require("./BaseFlowActionProcessor");
class ParallelFlowActionProcessor extends BaseFlowActionProcessor_1.BaseFlowActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return ParallelFlowActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            const flowService = typedi_1.Container.get(services_1.FlowService);
            let actions;
            let shareParameters = false;
            if (Array.isArray(this.options)) {
                actions = this.options;
            }
            else {
                actions = this.options.actions;
                shareParameters = this.options.shareParameters;
            }
            const snapshots = [];
            const promises = actions.map((action, index) => __awaiter(this, void 0, void 0, function* () {
                const iterationParams = this.getParameters(shareParameters, { index });
                snapshots[index] = yield flowService.executeAction(this.snapshot.source, this.snapshot.wd, action, this.context, iterationParams, this.snapshot);
            }));
            yield Promise.all(promises);
            // register snapshots in the order of their presence
            snapshots.forEach((childSnapshot) => {
                this.snapshot.registerChildActionSnapshot(childSnapshot);
            });
        });
    }
}
exports.ParallelFlowActionProcessor = ParallelFlowActionProcessor;
ParallelFlowActionProcessor.actionsValidationSchema = Joi.array().items(schemas_1.FBL_ACTION_SCHEMA.optional()).options({
    abortEarly: true,
});
ParallelFlowActionProcessor.validationSchema = Joi.alternatives(ParallelFlowActionProcessor.actionsValidationSchema, Joi.object()
    .keys({
    actions: ParallelFlowActionProcessor.actionsValidationSchema.required(),
    shareParameters: Joi.boolean(),
})
    .required()
    .options({
    allowUnknown: false,
    abortEarly: true,
}));
class ParallelFlowActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return ParallelFlowActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new ParallelFlowActionProcessor(options, context, snapshot, parameters);
    }
}
exports.ParallelFlowActionHandler = ParallelFlowActionHandler;
ParallelFlowActionHandler.metadata = {
    id: 'com.fireblink.fbl.flow.parallel',
    aliases: ['fbl.flow.parallel', 'flow.parallel', 'parallel', 'async', '||'],
    // We don't want to process options as a template to avoid unexpected behaviour inside nested actions
    skipTemplateProcessing: true,
};
//# sourceMappingURL=ParallelFlowActionHandler.js.map