"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RetryActionHandler = exports.RetryActionProcessor = void 0;
const models_1 = require("../../models");
const typedi_1 = require("typedi");
const Joi = require("joi");
const services_1 = require("../../services");
const schemas_1 = require("../../schemas");
const utils_1 = require("../../utils");
const errors_1 = require("../../errors");
class RetryActionProcessor extends models_1.ActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return RetryActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            const flowService = typedi_1.Container.get(services_1.FlowService);
            let retriesLeft = this.options.attempts;
            let childSnapshot;
            while (retriesLeft > 0) {
                // run try
                childSnapshot = yield flowService.executeAction(this.snapshot.source, this.snapshot.wd, this.options.action, this.context, this.parameters, this.snapshot);
                this.snapshot.registerChildActionSnapshot(childSnapshot);
                if (childSnapshot.successful) {
                    break;
                }
                retriesLeft--;
            }
            if (childSnapshot.successful) {
                this.snapshot.ignoreChildFailure = true;
            }
            if (!childSnapshot.successful && this.options.errorCode) {
                const code = childSnapshot.errorCode || errors_1.UNEXPECTED;
                utils_1.ContextUtil.assignTo(this.context, this.parameters, this.snapshot, this.options.errorCode.assignTo, code);
                utils_1.ContextUtil.pushTo(this.context, this.parameters, this.snapshot, this.options.errorCode.pushTo, code);
            }
        });
    }
}
exports.RetryActionProcessor = RetryActionProcessor;
RetryActionProcessor.validationSchema = Joi.object({
    action: schemas_1.FBL_ACTION_SCHEMA,
    attempts: Joi.number().required().min(1),
    errorCode: Joi.object({
        assignTo: schemas_1.FBL_ASSIGN_TO_SCHEMA,
        pushTo: schemas_1.FBL_PUSH_TO_SCHEMA,
    }).options({ abortEarly: true, allowUnknown: false }),
})
    .required()
    .options({
    abortEarly: true,
    allowUnknown: false,
});
class RetryActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return RetryActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new RetryActionProcessor(options, context, snapshot, parameters);
    }
}
exports.RetryActionHandler = RetryActionHandler;
RetryActionHandler.metadata = {
    id: 'com.fireblink.fbl.flow.retry',
    aliases: ['fbl.flow.retry', 'flow.retry', 'retry'],
    skipTemplateProcessing: true,
};
//# sourceMappingURL=RetryActionHandler.js.map