"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SequenceFlowActionHandler = exports.SequenceFlowActionProcessor = void 0;
const models_1 = require("../../models");
const Joi = require("joi");
const typedi_1 = require("typedi");
const services_1 = require("../../services");
const schemas_1 = require("../../schemas");
const BaseFlowActionProcessor_1 = require("./BaseFlowActionProcessor");
class SequenceFlowActionProcessor extends BaseFlowActionProcessor_1.BaseFlowActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return SequenceFlowActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            const flowService = typedi_1.Container.get(services_1.FlowService);
            let actions;
            let shareParameters = false;
            if (Array.isArray(this.options)) {
                actions = this.options;
            }
            else {
                actions = this.options.actions;
                shareParameters = this.options.shareParameters;
            }
            let index = 0;
            for (const action of actions) {
                const actionParameters = this.getParameters(shareParameters, { index });
                const childSnapshot = yield flowService.executeAction(this.snapshot.source, this.snapshot.wd, action, this.context, actionParameters, this.snapshot);
                this.snapshot.registerChildActionSnapshot(childSnapshot);
                index++;
                // stop processing after first failure
                if (!childSnapshot.successful) {
                    return;
                }
            }
        });
    }
}
exports.SequenceFlowActionProcessor = SequenceFlowActionProcessor;
SequenceFlowActionProcessor.actionsValidationSchema = Joi.array()
    .items(schemas_1.FBL_ACTION_SCHEMA.optional())
    .required()
    .options({ abortEarly: true });
SequenceFlowActionProcessor.validationSchema = Joi.alternatives(SequenceFlowActionProcessor.actionsValidationSchema, Joi.object()
    .keys({
    actions: SequenceFlowActionProcessor.actionsValidationSchema.required(),
    shareParameters: Joi.boolean(),
})
    .required()
    .options({
    allowUnknown: false,
    abortEarly: true,
}));
class SequenceFlowActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return SequenceFlowActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new SequenceFlowActionProcessor(options, context, snapshot, parameters);
    }
}
exports.SequenceFlowActionHandler = SequenceFlowActionHandler;
SequenceFlowActionHandler.metadata = {
    id: 'com.fireblink.fbl.flow.sequence',
    aliases: ['fbl.flow.sequence', 'flow.sequence', 'sequence', 'sync', '--'],
    // We don't want to process options as a template to avoid unexpected behaviour inside nested actions
    skipTemplateProcessing: true,
};
//# sourceMappingURL=SequenceFlowActionHandler.js.map