"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SwitchFlowActionHandler = exports.SwitchFlowActionProcessor = void 0;
const models_1 = require("../../models");
const typedi_1 = require("typedi");
const Joi = require("joi");
const services_1 = require("../../services");
const schemas_1 = require("../../schemas");
class SwitchFlowActionProcessor extends models_1.ActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return SwitchFlowActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            const flowService = typedi_1.Container.get(services_1.FlowService);
            // register masked options in the snapshot
            const masked = yield flowService.resolveOptionsWithNoHandlerCheck(this.context.ejsTemplateDelimiters.local, this.options.value, this.context, this.snapshot, this.parameters, true);
            this.snapshot.setOptions({
                value: masked,
                is: this.options.is,
            });
            const value = yield flowService.resolveOptionsWithNoHandlerCheck(this.context.ejsTemplateDelimiters.local, this.options.value, this.context, this.snapshot, this.parameters, false);
            let action;
            for (const is of Object.keys(this.options.is)) {
                if (is.toString() === value.toString()) {
                    action = this.options.is[is];
                    break;
                }
            }
            if (!action) {
                if (this.options.else) {
                    action = this.options.else;
                }
            }
            if (action) {
                this.snapshot.log(`Based on value: ${value} invoking handler.`);
                const childSnapshot = yield flowService.executeAction(this.snapshot.source, this.snapshot.wd, action, this.context, this.parameters, this.snapshot);
                this.snapshot.registerChildActionSnapshot(childSnapshot);
            }
            else {
                this.snapshot.log(`Unable to find handler for value: ${value}`, true);
            }
        });
    }
}
exports.SwitchFlowActionProcessor = SwitchFlowActionProcessor;
SwitchFlowActionProcessor.validationSchema = Joi.object({
    value: Joi.alternatives(Joi.string(), Joi.number(), Joi.boolean()).required(),
    is: Joi.object().pattern(/^/, schemas_1.FBL_ACTION_SCHEMA).min(1).required(),
    else: schemas_1.FBL_ACTION_SCHEMA.optional(),
})
    .required()
    .options({ abortEarly: true });
class SwitchFlowActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return SwitchFlowActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new SwitchFlowActionProcessor(options, context, snapshot, parameters);
    }
}
exports.SwitchFlowActionHandler = SwitchFlowActionHandler;
SwitchFlowActionHandler.metadata = {
    id: 'com.fireblink.fbl.flow.switch',
    aliases: ['fbl.flow.switch', 'flow.switch', 'switch', 'if', '?'],
    // we don't want to process templates inside options in a default way as it may cause processing of templates
    // inside nested actions, but we will need to process "value" as it supposed to use template.
    skipTemplateProcessing: true,
};
//# sourceMappingURL=SwitchFlowActionHandler.js.map