"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemplateFlowActionHandler = exports.TemplateFlowActionProcessor = void 0;
const models_1 = require("../../models");
const Joi = require("joi");
const services_1 = require("../../services");
const typedi_1 = require("typedi");
const js_yaml_1 = require("js-yaml");
const schemas_1 = require("../../schemas");
const errors_1 = require("../../errors");
class TemplateFlowActionProcessor extends models_1.ActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return TemplateFlowActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    validate() {
        const _super = Object.create(null, {
            validate: { get: () => super.validate }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield _super.validate.call(this);
            const action = js_yaml_1.safeLoad(this.options);
            try {
                Joi.assert(action, schemas_1.FBL_ACTION_SCHEMA);
            }
            catch (e) {
                throw new errors_1.ActionError(e.details.map((d) => d.message).join('\n'), errors_1.INVALID_CONFIGURATION);
            }
        });
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            const flowService = typedi_1.Container.get(services_1.FlowService);
            const action = js_yaml_1.safeLoad(this.options);
            const childSnapshot = yield flowService.executeAction(this.snapshot.source, this.snapshot.wd, action, this.context, this.parameters);
            this.snapshot.registerChildActionSnapshot(childSnapshot);
        });
    }
}
exports.TemplateFlowActionProcessor = TemplateFlowActionProcessor;
TemplateFlowActionProcessor.validationSchema = Joi.string().min(1).required().options({ abortEarly: true });
class TemplateFlowActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return TemplateFlowActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new TemplateFlowActionProcessor(options, context, snapshot, parameters);
    }
}
exports.TemplateFlowActionHandler = TemplateFlowActionHandler;
TemplateFlowActionHandler.metadata = {
    id: 'com.fireblink.fbl.flow.template',
    aliases: ['fbl.flow.template', 'flow.template', 'template', 'tpl'],
};
//# sourceMappingURL=TemplateFlowActionHandler.js.map