"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WhileActionHandler = exports.WhileActionProcessor = void 0;
const Joi = require("joi");
const typedi_1 = require("typedi");
const models_1 = require("../../models");
const services_1 = require("../../services");
const schemas_1 = require("../../schemas");
const BaseFlowActionProcessor_1 = require("./BaseFlowActionProcessor");
class WhileActionProcessor extends BaseFlowActionProcessor_1.BaseFlowActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return WhileActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    isShouldExecuteWithParameters(parameters) {
        return __awaiter(this, void 0, void 0, function* () {
            const flowService = typedi_1.Container.get(services_1.FlowService);
            if (this.snapshot.childFailure) {
                return false;
            }
            const value = yield flowService.resolveOptionsWithNoHandlerCheck(this.context.ejsTemplateDelimiters.local, this.options.value, this.context, this.snapshot, parameters, false);
            if (this.options.is !== undefined) {
                const is = yield flowService.resolveOptionsWithNoHandlerCheck(this.context.ejsTemplateDelimiters.local, this.options.is, this.context, this.snapshot, parameters, false);
                return value.toString() === is.toString();
            }
            else {
                const not = yield flowService.resolveOptionsWithNoHandlerCheck(this.context.ejsTemplateDelimiters.local, this.options.not, this.context, this.snapshot, parameters, false);
                return value.toString() !== not.toString();
            }
        });
    }
    /**
     * @inheritdoc
     */
    isShouldExecute() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.isShouldExecuteWithParameters(this.parameters);
        });
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            const flowService = typedi_1.Container.get(services_1.FlowService);
            let index = 0;
            let actionParameters = this.getParameters(this.options.shareParameters, { index });
            let execute = yield this.isShouldExecuteWithParameters(actionParameters);
            while (execute) {
                const childSnapshot = yield flowService.executeAction(this.snapshot.source, this.snapshot.wd, this.options.action, this.context, actionParameters, this.snapshot);
                this.snapshot.registerChildActionSnapshot(childSnapshot);
                index++;
                actionParameters = this.getParameters(this.options.shareParameters, { index });
                execute = yield this.isShouldExecuteWithParameters(actionParameters);
            }
        });
    }
}
exports.WhileActionProcessor = WhileActionProcessor;
WhileActionProcessor.validationSchema = Joi.object({
    shareParameters: Joi.boolean(),
    value: Joi.alternatives(Joi.string(), Joi.number(), Joi.boolean()).required(),
    not: Joi.alternatives(Joi.string(), Joi.number(), Joi.boolean()),
    is: Joi.alternatives(Joi.string(), Joi.number(), Joi.boolean()),
    action: schemas_1.FBL_ACTION_SCHEMA,
})
    .xor('not', 'is')
    .required()
    .options({ abortEarly: true });
class WhileActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return WhileActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new WhileActionProcessor(options, context, snapshot, parameters);
    }
}
exports.WhileActionHandler = WhileActionHandler;
WhileActionHandler.metadata = {
    id: 'com.fireblink.fbl.flow.while',
    aliases: ['fbl.flow.while', 'flow.while', 'while'],
    // we don't want to process templates inside options in a default way as it may cause processing of templates
    // inside nested actions, but we will need to process "value" as it supposed to use template.
    skipTemplateProcessing: true,
};
//# sourceMappingURL=WhileActionHandler.js.map