"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CopyPathActionHandler = exports.CopyPathActionProcessor = void 0;
const models_1 = require("../../models");
const Joi = require("joi");
const utils_1 = require("../../utils");
const path_1 = require("path");
class CopyPathActionProcessor extends models_1.ActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return CopyPathActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            let from = utils_1.FSUtil.getAbsolutePath(this.options.from, this.snapshot.wd);
            if (this.options.from.endsWith(path_1.sep)) {
                from += path_1.sep;
            }
            let to = utils_1.FSUtil.getAbsolutePath(this.options.to, this.snapshot.wd);
            if (this.options.to.endsWith(path_1.sep)) {
                to += path_1.sep;
            }
            yield utils_1.FSUtil.copy(from, to);
        });
    }
}
exports.CopyPathActionProcessor = CopyPathActionProcessor;
CopyPathActionProcessor.validationSchema = Joi.object({
    from: Joi.string().min(1).required(),
    to: Joi.string().min(1).required(),
})
    .required()
    .options({
    abortEarly: true,
});
class CopyPathActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return CopyPathActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new CopyPathActionProcessor(options, context, snapshot, parameters);
    }
}
exports.CopyPathActionHandler = CopyPathActionHandler;
CopyPathActionHandler.metadata = {
    id: 'com.fireblink.fbl.fs.copy',
    aliases: ['fbl.fs.copy', 'fbl.fs.cp', 'fs.cp', 'fs.copy', 'copy', 'cp'],
};
//# sourceMappingURL=CopyPathActionHandler.js.map