"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FindActionHandler = exports.FindActionProcessor = void 0;
const models_1 = require("../../models");
const Joi = require("joi");
const utils_1 = require("../../utils");
const schemas_1 = require("../../schemas");
const errors_1 = require("../../errors");
class FindActionProcessor extends models_1.ActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return FindActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            let result = yield utils_1.FSUtil.findFilesByMasks(this.options.include, this.options.exclude || [], this.snapshot.wd);
            if (this.options.result.baseDir) {
                const baseDir = utils_1.FSUtil.getAbsolutePath(this.options.result.baseDir, this.snapshot.wd);
                result = result.map((p) => {
                    if (!p.startsWith(baseDir)) {
                        throw new errors_1.ActionError(`Unable to find baseDir "${this.options.result.baseDir}" in path "${p}"`, '500');
                    }
                    return p.substring(baseDir.length + 1);
                });
            }
            /* istanbul ignore else */
            if (this.options.result.assignTo) {
                utils_1.ContextUtil.assignTo(this.context, this.parameters, this.snapshot, this.options.result.assignTo, result);
            }
            /* istanbul ignore else */
            if (this.options.result.pushTo) {
                utils_1.ContextUtil.pushTo(this.context, this.parameters, this.snapshot, this.options.result.pushTo, result);
            }
        });
    }
}
exports.FindActionProcessor = FindActionProcessor;
FindActionProcessor.validationSchema = Joi.object({
    include: Joi.array().items(Joi.string().min(1).required()).min(1).required(),
    exclude: Joi.array().items(Joi.string().min(1).required()),
    result: Joi.object({
        baseDir: Joi.string().min(1),
        assignTo: schemas_1.FBL_ASSIGN_TO_SCHEMA,
        pushTo: schemas_1.FBL_PUSH_TO_SCHEMA,
    })
        .or('assignTo', 'pushTo')
        .required(),
})
    .required()
    .options({
    abortEarly: true,
});
class FindActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return FindActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new FindActionProcessor(options, context, snapshot, parameters);
    }
}
exports.FindActionHandler = FindActionHandler;
FindActionHandler.metadata = {
    id: 'com.fireblink.fbl.fs.find',
    aliases: ['fbl.fs.find', 'fs.find', 'find'],
};
//# sourceMappingURL=FindActionHandler.js.map