"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemovePathActionHandler = exports.RemovePathActionProcessor = void 0;
const models_1 = require("../../models");
const Joi = require("joi");
const utils_1 = require("../../utils");
class RemovePathActionProcessor extends models_1.ActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return RemovePathActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            const path = utils_1.FSUtil.getAbsolutePath(this.options, this.snapshot.wd);
            yield utils_1.FSUtil.remove(path);
        });
    }
}
exports.RemovePathActionProcessor = RemovePathActionProcessor;
RemovePathActionProcessor.validationSchema = Joi.string().min(1).required();
class RemovePathActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return RemovePathActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new RemovePathActionProcessor(options, context, snapshot, parameters);
    }
}
exports.RemovePathActionHandler = RemovePathActionHandler;
RemovePathActionHandler.metadata = {
    id: 'com.fireblink.fbl.fs.remove',
    aliases: ['fbl.fs.remove', 'fbl.fs.rm', 'fs.rm', 'fs.remove', 'rm -rf', 'remove', 'rm'],
};
//# sourceMappingURL=RemovePathActionHandler.js.map