"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WriteToFileActionHandler = exports.WriteToFileActionProcessor = void 0;
const models_1 = require("../../models");
const Joi = require("joi");
const fs_1 = require("fs");
const util_1 = require("util");
const utils_1 = require("../../utils");
const path_1 = require("path");
const typedi_1 = require("typedi");
const services_1 = require("../../services");
const schemas_1 = require("../../schemas");
class WriteToFileActionProcessor extends models_1.ActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return WriteToFileActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            let file;
            if (this.options.path) {
                file = utils_1.FSUtil.getAbsolutePath(this.options.path, this.snapshot.wd);
            }
            else {
                file = yield typedi_1.Container.get(services_1.TempPathsRegistry).createTempFile(true);
            }
            // create folders structure if needed
            yield utils_1.FSUtil.mkdirp(path_1.dirname(file));
            let content = this.options.content;
            if (this.options.contentFromFile) {
                const flowService = typedi_1.Container.get(services_1.FlowService);
                const absolutePath = utils_1.FSUtil.getAbsolutePath(this.options.contentFromFile, this.snapshot.wd);
                content = yield utils_1.FSUtil.readTextFile(absolutePath);
                const wd = path_1.dirname(absolutePath);
                // resolve with global template delimiter first
                content = yield flowService.resolveTemplate(this.context.ejsTemplateDelimiters.global, content, this.context, this.snapshot, this.parameters, {}, wd);
                // resolve local template delimiter
                content = yield flowService.resolveTemplate(this.context.ejsTemplateDelimiters.local, content, this.context, this.snapshot, this.parameters, {}, wd);
            }
            this.snapshot.log(`Writing content to a file: ${file}`);
            yield util_1.promisify(fs_1.writeFile)(file, content, 'utf8');
            /* istanbul ignore else */
            if (this.options.assignPathTo) {
                utils_1.ContextUtil.assignTo(this.context, this.parameters, this.snapshot, this.options.assignPathTo, file);
            }
            /* istanbul ignore else */
            if (this.options.pushPathTo) {
                utils_1.ContextUtil.pushTo(this.context, this.parameters, this.snapshot, this.options.pushPathTo, file);
            }
        });
    }
}
exports.WriteToFileActionProcessor = WriteToFileActionProcessor;
WriteToFileActionProcessor.validationSchema = Joi.object({
    path: Joi.string().min(1),
    assignPathTo: schemas_1.FBL_ASSIGN_TO_SCHEMA,
    pushPathTo: schemas_1.FBL_PUSH_TO_SCHEMA,
    contentFromFile: Joi.string().min(1),
    content: Joi.alternatives(Joi.number(), Joi.string()),
})
    .or('path', 'assignPathTo')
    .xor('content', 'contentFromFile')
    .required()
    .options({ abortEarly: true });
class WriteToFileActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return WriteToFileActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new WriteToFileActionProcessor(options, context, snapshot, parameters);
    }
}
exports.WriteToFileActionHandler = WriteToFileActionHandler;
WriteToFileActionHandler.metadata = {
    id: 'com.fireblink.fbl.fs.file.write',
    aliases: ['fbl.fs.file.write', 'fs.file.write', 'file.write', '->'],
};
//# sourceMappingURL=WriteToFileActionHandler.js.map