"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfirmActionHandler = exports.ConfirmActionProcessor = void 0;
const models_1 = require("../../models");
const Joi = require("joi");
const utils_1 = require("../../utils");
const BasePromptActionProcessor_1 = require("./BasePromptActionProcessor");
const schemas_1 = require("../../schemas");
class ConfirmActionProcessor extends BasePromptActionProcessor_1.BasePromptActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return ConfirmActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            const value = yield this.prompt({
                type: 'confirm',
                initial: this.options.default,
                message: this.options.message,
            });
            /* istanbul ignore else */
            if (this.options.assignResponseTo) {
                utils_1.ContextUtil.assignTo(this.context, this.parameters, this.snapshot, this.options.assignResponseTo, value);
            }
            /* istanbul ignore else */
            if (this.options.pushResponseTo) {
                utils_1.ContextUtil.pushTo(this.context, this.parameters, this.snapshot, this.options.pushResponseTo, value);
            }
        });
    }
}
exports.ConfirmActionProcessor = ConfirmActionProcessor;
ConfirmActionProcessor.validationSchema = Joi.object({
    message: Joi.string().required().min(1),
    default: Joi.boolean(),
    assignResponseTo: schemas_1.FBL_ASSIGN_TO_SCHEMA,
    pushResponseTo: schemas_1.FBL_PUSH_TO_SCHEMA,
})
    .or('assignResponseTo', 'pushResponseTo')
    .required();
class ConfirmActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return ConfirmActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new ConfirmActionProcessor(options, context, snapshot, parameters);
    }
}
exports.ConfirmActionHandler = ConfirmActionHandler;
ConfirmActionHandler.metadata = {
    id: 'com.fireblink.fbl.cli.prompts.confirm',
    aliases: ['fbl.cli.prompts.confirm', 'cli.prompts.confirm', 'prompts.confirm', 'confirm'],
};
//# sourceMappingURL=ConfirmActionHandler.js.map