"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PromptActionHandler = exports.PromptActionProcessor = void 0;
const models_1 = require("../../models");
const jsonschema_1 = require("jsonschema");
const Joi = require("joi");
const utils_1 = require("../../utils");
const BasePromptActionProcessor_1 = require("./BasePromptActionProcessor");
const schemas_1 = require("../../schemas");
class PromptActionProcessor extends BasePromptActionProcessor_1.BasePromptActionProcessor {
    /**
     * @inheritdoc
     */
    getValidationSchema() {
        return PromptActionProcessor.validationSchema;
    }
    /**
     * @inheritdoc
     */
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            let type = 'text';
            let float;
            let min;
            let max;
            /* istanbul ignore else */
            if (this.options.schema && this.options.schema.type !== 'string') {
                type = 'number';
                /* istanbul ignore else */
                if (this.options.schema.type === 'number') {
                    float = true;
                }
                /* istanbul ignore else */
                if (this.options.schema.minimum) {
                    min = this.options.schema.minimum;
                }
                /* istanbul ignore else */
                if (this.options.schema.maximum) {
                    max = this.options.schema.maximum;
                }
            }
            const validator = new jsonschema_1.Validator();
            let value = yield this.prompt({
                type: type,
                float: float,
                min: min,
                max: max,
                initial: this.options.default,
                style: this.options.password ? 'password' : 'default',
                message: this.options.message,
                validate: (v) => {
                    if (this.options.schema) {
                        if (this.options.schema.type !== 'string' && !isNaN(v)) {
                            v = Number(v);
                        }
                        const result = validator.validate(v, this.options.schema);
                        if (!result.valid) {
                            return result.errors.map((e) => `value ${e.message}`).join('\n');
                        }
                    }
                    return true;
                },
            });
            if (this.options.schema && this.options.schema.type !== 'string' && !isNaN(value)) {
                value = Number(value);
            }
            /* istanbul ignore else */
            if (this.options.assignResponseTo) {
                utils_1.ContextUtil.assignTo(this.context, this.parameters, this.snapshot, this.options.assignResponseTo, value);
            }
            /* istanbul ignore else */
            if (this.options.pushResponseTo) {
                utils_1.ContextUtil.pushTo(this.context, this.parameters, this.snapshot, this.options.pushResponseTo, value);
            }
        });
    }
}
exports.PromptActionProcessor = PromptActionProcessor;
PromptActionProcessor.validationSchema = Joi.object({
    message: Joi.string().required().min(1),
    password: Joi.boolean(),
    default: Joi.alternatives(Joi.string(), Joi.number()),
    schema: Joi.object({
        type: Joi.string().valid('string', 'integer', 'number').required(),
        // when type is "string"
        pattern: Joi.string().min(1),
        minLength: Joi.number().min(0),
        maxLength: Joi.number().min(1),
        // when type is "number" or "integer":
        multipleOf: Joi.number(),
        minimum: Joi.number(),
        maximum: Joi.number(),
        exclusiveMinimum: Joi.boolean(),
        exclusiveMaximum: Joi.boolean(),
    }),
    assignResponseTo: schemas_1.FBL_ASSIGN_TO_SCHEMA,
    pushResponseTo: schemas_1.FBL_PUSH_TO_SCHEMA,
})
    .or('assignResponseTo', 'pushResponseTo')
    .required();
class PromptActionHandler extends models_1.ActionHandler {
    /**
     * @inheritdoc
     */
    getMetadata() {
        return PromptActionHandler.metadata;
    }
    /**
     * @inheritdoc
     */
    getProcessor(options, context, snapshot, parameters) {
        return new PromptActionProcessor(options, context, snapshot, parameters);
    }
}
exports.PromptActionHandler = PromptActionHandler;
PromptActionHandler.metadata = {
    id: 'com.fireblink.fbl.cli.prompts.prompt',
    aliases: ['fbl.cli.prompts.prompt', 'cli.prompts.prompt', 'prompts.prompt', 'prompt'],
};
//# sourceMappingURL=PromptActionHandler.js.map