"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var CLIService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.CLIService = void 0;
const typedi_1 = require("typedi");
const commander = require("commander");
const object_collider_1 = require("object-collider");
const colors = require("colors");
const index_1 = require("./index");
const utils_1 = require("../utils");
const TempPathsRegistry_1 = require("./TempPathsRegistry");
const table_1 = require("table");
const models_1 = require("../models");
const prompts = require('prompts');
const cliui = require('cliui');
let CLIService = CLIService_1 = class CLIService {
    get fbl() {
        return typedi_1.default.get(index_1.FBLService);
    }
    get flowService() {
        return typedi_1.default.get(index_1.FlowService);
    }
    get tempPathsRegistry() {
        return typedi_1.default.get(TempPathsRegistry_1.TempPathsRegistry);
    }
    get logService() {
        return typedi_1.default.get(index_1.LogService);
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            this.globalConfig = {
                plugins: [],
                context: {
                    ctx: {},
                    secrets: {},
                    parameters: {},
                },
                report: {
                    options: {},
                },
                http: {
                    headers: {},
                },
                other: {},
            };
            this.globalConfig.plugins.push(...CLIService_1.CORE_PLUGINS);
            yield this.parseParameters();
            if (this.globalConfig.other.noColors) {
                colors.disable();
            }
            else {
                colors.enable();
            }
            if (commander.unsafePlugins) {
                this.fbl.allowUnsafePlugins = true;
            }
            if (commander.unsafeFlows) {
                this.fbl.allowUnsafeFlows = true;
            }
            if (this.globalConfig.report.output) {
                // enable debug mode when report generation is requested
                this.flowService.debug = true;
            }
            yield this.registerPlugins();
            if (this.globalConfig.report.output) {
                if (!this.fbl.getReporter(this.globalConfig.report.type)) {
                    console.error(`Error: Unable to find reporter: ${this.globalConfig.report.type}`);
                    commander.outputHelp();
                    process.exit(1);
                }
            }
            const context = yield this.prepareContext();
            const parameters = {
                parameters: this.globalConfig.context.parameters,
            };
            if (this.globalConfig.other && this.globalConfig.other.globalTemplateDelimiter) {
                context.ejsTemplateDelimiters.global = this.globalConfig.other.globalTemplateDelimiter;
            }
            if (this.globalConfig.other && this.globalConfig.other.localTemplateDelimiter) {
                context.ejsTemplateDelimiters.local = this.globalConfig.other.localTemplateDelimiter;
            }
            const source = utils_1.FSUtil.getAbsolutePath(this.flowFilePath, process.cwd());
            const flow = yield this.flowService.readFlowFromFile({
                path: source,
                http: {
                    headers: this.globalConfig.http && this.globalConfig.http.headers,
                },
                target: this.flowTarget,
                cache: this.globalConfig.other.useCache,
            }, context, new models_1.ActionSnapshot(source, '', {}, process.cwd(), 0, {}), parameters);
            let initialContextState;
            let finalContextState;
            if (this.globalConfig.report.output) {
                initialContextState = JSON.parse(JSON.stringify(utils_1.ContextUtil.toBase(context)));
            }
            const snapshot = yield this.fbl.execute(source, flow.wd, flow.flow, context, parameters);
            if (this.globalConfig.report.output) {
                finalContextState = JSON.parse(JSON.stringify(utils_1.ContextUtil.toBase(context)));
            }
            if (this.globalConfig.report.output) {
                const report = {
                    context: {
                        initial: initialContextState,
                        final: finalContextState,
                    },
                    snapshot: snapshot,
                };
                // generate report
                yield this.fbl
                    .getReporter(this.globalConfig.report.type)
                    .generate(this.globalConfig.report.output, this.globalConfig.report.options, report);
            }
            // remove all temp files and folders
            yield this.tempPathsRegistry.cleanup();
            if (!snapshot.successful) {
                throw new Error('Execution failed.');
            }
            else {
                if (context.summary.length) {
                    CLIService_1.printSummary(context.summary);
                }
            }
        });
    }
    /**
     * Print summary
     * @param {ISummaryRecord[]} records
     */
    static printSummary(records) {
        const includeDuration = records.find((r) => !!r.duration);
        const head = ['Title'.bold, 'Status'.bold];
        /* istanbul ignore else */
        if (includeDuration) {
            head.push('Duration'.bold);
        }
        console.log();
        console.log(table_1.table([
            head,
            ...records.map((r) => {
                let status = r.status.trim();
                if (['created', 'updated', 'passed', 'success', 'ok', 'yes'].indexOf(status.toLowerCase()) >= 0) {
                    status = status.green;
                }
                if (['deleted', 'failed', 'failure', 'error', 'no'].indexOf(status.toLowerCase()) >= 0) {
                    status = status.red;
                }
                if (['ignored', 'skipped', 'none'].indexOf(status.toLowerCase()) >= 0) {
                    status = status.yellow;
                }
                const row = [r.title, status];
                /* istanbul ignore else */
                if (includeDuration) {
                    row.push(r.duration);
                }
                return row;
            }),
        ]));
    }
    /**
     * Parse parameters passed via CLI
     */
    parseParameters() {
        return __awaiter(this, void 0, void 0, function* () {
            const assign = [];
            const reportOptions = [];
            const options = [
                {
                    flags: '-p --plugin <file>',
                    description: ['Plugin file.'],
                    fn: (val) => {
                        this.globalConfig.plugins.push(val);
                    },
                },
                {
                    flags: '-a --assign <key=value>',
                    description: [
                        'Assign key value pair as default values for cxt, secrets or parameters.',
                        'Expected key format: $.<ctx | secrets | parameters>[.<parent>][.child][...]',
                        'Note: ',
                        ' - if value is started with "@" it will be treated as YAML file and content will be loaded from it; use @@ to escape first character to avoid this behavior',
                        ' - if value is started with "%" it will be treated as JSON string and content will be loaded from it; use %% to escape first character to avoid this behavior',
                    ],
                    fn: (val) => {
                        assign.push(val);
                    },
                },
                {
                    flags: '-o --output <file>',
                    description: ['Execution report path'],
                },
                {
                    flags: '-r --report <name>',
                    description: ['Execution report format'],
                },
                {
                    flags: '-t --target <path>',
                    description: ['Custom relative path inside the packaged flow (tarball).'],
                },
                {
                    flags: '--report-option <key=value>',
                    description: [
                        'Key value pair of report option',
                        'Note: ',
                        ' - if value is started with "@" it will be treated as YAML file and content will be loaded from it; use @@ to escape first character to avoid this behavior',
                        ' - if value is started with "%" it will be treated as JSON string and content will be loaded from it; use %% to escape first character to avoid this behavior',
                    ],
                    fn: (val) => {
                        reportOptions.push(val);
                    },
                },
                {
                    flags: '--unsafe-plugins',
                    description: [
                        'If provided incompatible plugins will still be registered and be available for use,',
                        'but may lead to unexpected results or errors.',
                    ],
                },
                {
                    flags: '--unsafe-flows',
                    description: [
                        'If provided incompatible flow requirements will be ignored,',
                        'but may lead to unexpected results or errors.',
                    ],
                },
                {
                    flags: '--no-colors',
                    description: ['Remove colors from output. Make it boring.'],
                },
                {
                    flags: '--global-template-delimiter <delimiter>',
                    description: ['Global EJS template delimiter. Default: $'],
                },
                {
                    flags: '--local-template-delimiter <delimiter>',
                    description: ['Local EJS template delimiter. Default: %'],
                },
                {
                    flags: '--http-header <header>',
                    description: ['Custom HTTP headers to send with GET request to fetch flow from remote location.'],
                    fn: (val) => {
                        const name = val.split(':')[0];
                        this.globalConfig.http.headers[name] = val.substring(name.length + 1).trimLeft();
                    },
                },
                {
                    flags: '--use-cache',
                    description: [
                        'Cache remote package into $FBL_HOME/cache folder.',
                        'If package already exists inside cache dir - it will be used and no HTTP requests will be made.',
                        '',
                        'Note: this option will only work for path provided to CLI.',
                        'Option has no affect on "attachment" actions inside the flow itself.',
                    ],
                },
                {
                    flags: '--verbose',
                    description: ['Output additional logs.'],
                },
                {
                    flags: '-h --help',
                    description: ['Output usage information.'],
                    fn: () => {
                        this.printHelp(options);
                        process.exit(0);
                    },
                },
            ];
            // prepare commander
            commander
                .version(require('../../../package.json').version)
                .usage('[options] [path]')
                .arguments('[path]')
                .action((path, opts) => {
                opts.path = path;
            });
            // register options
            options.forEach((option) => {
                commander.option(option.flags, option.description.join(' '), option.fn);
            });
            commander.on('--help', () => {
                this.printHelp(options);
            });
            // parse environment variables
            yield commander.parseAsync(process.argv);
            for (const v of assign) {
                yield this.convertKVPair(v, this.globalConfig.context);
            }
            for (const v of reportOptions) {
                yield this.convertKVPair(v, this.globalConfig.report.options);
            }
            if (!commander.path) {
                console.error('Error: flow descriptor file/url was not provided.\n');
                commander.outputHelp();
                process.exit(1);
            }
            this.globalConfig.report.output = commander.output || this.globalConfig.report.output;
            this.globalConfig.report.type = commander.report || this.globalConfig.report.type;
            if (this.globalConfig.report.output && !this.globalConfig.report.type) {
                console.error('Error: --report parameter is missing.');
                commander.outputHelp();
                process.exit(1);
            }
            this.flowFilePath = commander.path;
            this.globalConfig.other.noColors = !commander.colors;
            this.globalConfig.other.useCache = commander.useCache;
            this.globalConfig.other.allowUnsafePlugins = commander.unsafePlugins;
            this.globalConfig.other.allowUnsafeFlows = commander.unsafeFlows;
            this.flowTarget = commander.target;
            if (commander.globalTemplateDelimiter) {
                this.globalConfig.other.globalTemplateDelimiter = commander.globalTemplateDelimiter;
            }
            if (commander.localTemplateDelimiter) {
                this.globalConfig.other.localTemplateDelimiter = commander.localTemplateDelimiter;
            }
            if (commander.verbose) {
                this.logService.enableInfoLogs();
            }
        });
    }
    /**
     * Output help to stdout
     * @param options
     */
    printHelp(options) {
        const allOptions = [
            ...options,
            {
                flags: '-V, --version',
                description: ['Output the version number'],
            },
        ];
        let maxFlagsLength = 0;
        let maxDescriptionLength = 0;
        allOptions.forEach((option) => {
            maxFlagsLength = Math.max(maxFlagsLength, option.flags.length);
            for (const line of option.description) {
                maxDescriptionLength = Math.max(maxDescriptionLength, line.length);
            }
        });
        const ui = cliui();
        ui.div('Usage: fbl [options] <file or url>');
        ui.div({
            text: 'Options:',
            padding: [1, 0, 1, 0],
        });
        allOptions.forEach((option) => {
            ui.div({
                text: option.flags,
                width: maxFlagsLength + 4,
                padding: [0, 2, 0, 2],
            }, {
                text: option.description.join('\n'),
                width: maxDescriptionLength,
            });
        });
        console.log(ui.toString());
    }
    /**
     * Register plugins
     */
    registerPlugins() {
        return __awaiter(this, void 0, void 0, function* () {
            const plugins = [];
            for (const nameOrPath of this.globalConfig.plugins) {
                const plugin = yield index_1.FBLService.requirePlugin(nameOrPath, process.cwd());
                plugins.push(plugin);
            }
            this.fbl.registerPlugins(plugins);
            yield this.fbl.validatePlugins(process.cwd());
        });
    }
    /**
     * Prepare context
     * @return {Promise<IContext>}
     */
    prepareContext() {
        return __awaiter(this, void 0, void 0, function* () {
            const context = utils_1.ContextUtil.generateEmptyContext();
            context.ctx = this.globalConfig.context.ctx;
            context.secrets = this.globalConfig.context.secrets;
            return context;
        });
    }
    /**
     * Convert key=value pairs into object
     * @param {string} kv key/value pair string
     * @param target
     * @param secret
     */
    convertKVPair(kv, target) {
        return __awaiter(this, void 0, void 0, function* () {
            const chunks = kv.split('=');
            if (chunks.length !== 2) {
                const secret = chunks[0].startsWith('$.secrets.');
                chunks[1] = (yield prompts({
                    type: 'text',
                    name: 'value',
                    style: secret ? 'password' : 'default',
                    message: `${chunks[0]}: `,
                })).value;
            }
            let value;
            let isObj = false;
            const path = chunks[0];
            try {
                if (chunks[1][0] === '@' && chunks[1][1] !== '@') {
                    const file = chunks[1].substring(1);
                    value = yield utils_1.FSUtil.readYamlFromFile(file);
                    // validate file content to be object
                    isObj = object_collider_1.isObject(value);
                }
                else {
                    value = chunks[1];
                    if ((value[0] === '@' || value[0] === '%') && value[0] === value[1]) {
                        value = value.substring(1);
                    }
                    else if (value[0] === '%') {
                        value = JSON.parse(value.substring(1));
                        isObj = object_collider_1.isObject(value);
                    }
                }
                if (path === '$') {
                    if (isObj) {
                        object_collider_1.collide(target, value);
                    }
                    else {
                        throw new Error('Unable to assign non-object value to root path "$"');
                    }
                }
                else {
                    const parentPath = utils_1.ContextUtil.getParentPath(path);
                    if (parentPath !== '$') {
                        utils_1.ContextUtil.instantiateObjectPath(target, parentPath);
                    }
                    const parent = utils_1.ContextUtil.getValueAtPath(target, parentPath);
                    object_collider_1.collideUnsafe(parent, { [path.split('.').pop()]: value });
                }
            }
            catch (e) {
                console.error(`Unable to assign value for path "${path}" based on string: ${kv}`);
                throw e;
            }
        });
    }
};
CLIService.CORE_PLUGINS = [
    __dirname + '/../plugins/context',
    __dirname + '/../plugins/exec',
    __dirname + '/../plugins/flow',
    __dirname + '/../plugins/fs',
    __dirname + '/../plugins/prompts',
    __dirname + '/../plugins/reporters',
    __dirname + '/../plugins/templateUtilities',
];
CLIService = CLIService_1 = __decorate([
    typedi_1.Service()
], CLIService);
exports.CLIService = CLIService;
//# sourceMappingURL=CLIService.js.map