/// <reference types="node" />
export declare class FSUtil {
    /**
     * Check if path exists
     * @param {string} path
     * @return {Promise<boolean>}
     */
    static exists(path: string): Promise<boolean>;
    /**
     * Check if path is actually a URL
     * @param path
     */
    static isURL(path: string): boolean;
    /**
     * Check if path is absolute
     * @param {string} path
     * @return {boolean}
     */
    static isAbsolute(path: string): boolean;
    /**
     * Find files by mask
     * @param {string[]} masks
     * @param {string[]} ignore
     * @param {string} wd
     * @return {Promise<string[]>}
     */
    static findFilesByMasks(masks: string[], ignore: string[], wd: string): Promise<string[]>;
    /**
     * Get absolute based on current working directory
     * @param {string} path
     * @param {string} wd Working Directory
     * @return {string}
     */
    static getAbsolutePath(path: string, wd: string): string;
    static readFile(file: string): Promise<Buffer>;
    /**
     * Get text file content
     * @param {string} file
     * @param {BufferEncoding} [encoding]
     * @return {Promise<string>}
     */
    static readTextFile(file: string, encoding?: BufferEncoding): Promise<string>;
    /**
     * Create directory and all its parents recursively
     * @param {string} path
     * @return {Promise<void>}
     */
    static mkdirp(path: string): Promise<void>;
    /**
     * Remove files and folders at given path recursively
     * @param {string} path
     * @param {boolean} skipNotFound do not rise exception if file no longer exists
     * @return {Promise<void>}
     */
    static remove(path: string, skipNotFound?: boolean): Promise<void>;
    /**
     * Check if path represents a directory
     * @param {string} path
     * @return {Promise<boolean>}
     */
    static isDirectory(path: string): Promise<boolean>;
    /**
     * Move file/folder
     * @param {string} from
     * @param {string} to
     * @return {Promise<void>}
     */
    static move(from: string, to: string): Promise<void>;
    /**
     * Copy file/folder
     * @param {string} from
     * @param {string} to
     * @return {Promise<void>}
     */
    static copy(from: string, to: string): Promise<void>;
    /**
     * Read and parse yaml file
     * @param {string} file
     * @returns {Promise<any>}
     */
    static readYamlFromFile(file: string): Promise<any>;
}
